@@ -2,7 +2,7 @@
 #
 # Converts WikiText files into docbook.
 #
-# Copyright (c) 2001, 2002 David Merrill <david@lupercalia.net>.
+# Copyright (c) 2001, 2002, 2003 David Merrill <david@lupercalia.net>.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -58,6 +58,7 @@ sub Reset {
     $dbfile = '';
     $verbose = 0;
     $doctype = 0;
+    $articleclass = '';
     $nonet = 0;
 
     # These maintain state
@@ -85,7 +86,7 @@ sub new {
 }
 
 sub ProcessFile {
-    ($self, $txtfile, $dbfile, $verbose, $doctype, $nonet, $encoding) = @_;
+    ($self, $txtfile, $dbfile, $verbose, $doctype, $articleclass, $nonet, $encoding) = @_;
 
     # Read from STDIN if no input file given
     # 
@@ -112,30 +113,41 @@ sub ProcessFile {
     #
     $encoding = 'ISO-8859-1' unless ($encoding);
     if ($doctype eq 'XML') {
-        print "Adding XML DOCTYPE and article tags." if ($verbose);
+        print "Adding XML DOCTYPE and article tags\n" if ($verbose);
         $buf = '<?xml version="1.0" encoding="' . $encoding . '" standalone="no"?>' . "\n";
         $buf .= '<!DOCTYPE article PUBLIC "-//OASIS//DTD DocBook XML V4.1.2//EN"' . "\n";
         $buf .= '    "http://docbook.org/xml/4.1.2/docbookx.dtd"';
         $buf .= "\[\n";
-
- 
         $buf .= '<!ENTITY % ISOnum PUBLIC' . "\n";
         $buf .= '  "ISO 8879:1986//ENTITIES Numeric and Special Graphic//EN//XML"' . "\n";
         $buf .= '  "http://docbook.org/xml/4.1.2/ent/iso-num.ent">' . "\n";
         $buf .= '  %ISOnum;' . "\n";
         $buf .= "\]\>\n";
         $buf .= "\n";
-        $buf .= '<article>' . "\n";
+        if ($articleclass) {
+		print "Setting article class to $articleclass\n" if ($verbose);
+		$buf .= "<article class='$articleclass'>\n";
+	} else {
+		$buf .= '<article>' . "\n";
+	}
     } elsif ($doctype eq 'SGML') {
-        print "Adding SGML DOCTYPE and article tags." if ($verbose);
+        print "Adding SGML DOCTYPE and article tags\n" if ($verbose);
         $buf = '<!DOCTYPE article PUBLIC "-//OASIS//DTD DocBook V4.1//EN">' . "\n";
-        $buf .= '<article>' . "\n";
+        if ($articleclass) {
+		$buf .= "<article class='$articleclass'>\n";
+	} else {
+		$buf .= '<article>' . "\n";
+	}
     }
 
     # read in the text file
     #
     while ($originalline = <$fh>) {
-        ProcessLine($foo, $originalline);
+    	chomp($originalline);
+    	$originalline =~ s/\x0a//g;;
+    	$originalline =~ s/\x0d//g;;
+	print "Read line $originalline\n" if ($verbose);
+	ProcessLine($foo, $originalline);
         print $outfh "$buf";
         $buf = '';
     }
@@ -213,21 +225,26 @@ sub ProcessLine {
 
         # namespaces are handled differently
         #
-        print "$link\n" if ($verbose);
+        print "Link to $link\n" if ($verbose);
 
         if ($link =~ /^http:\/\//) {
             $line =~ s/\[\[.*?\]\]/<ulink url='$link'><citetitle>$linkname<\/citetitle><\/ulink>/;
         } elsif ($link =~ /^link:/) {
             $link =~ s/^link://;
             $linkname =~ s/^link://;
-            $line =~ s/\[\[.*?\]\]/<xref linkend='$link' endterm='$link'\>\<\/xref\>/;
+            $line =~ s/\[\[.*?\]\]/<xref linkend='$link' endterm='$link-title'\/\>/;
+        } elsif ($link =~ /^ftp:/) {
+            $line =~ s/\[\[.*?\]\]/<ulink url='$link'><citetitle>$linkname<\/citetitle><\/ulink>/;
+        } elsif ($link =~ /^news:/) {
+            $linkname =~ s/^news:\/\///;
+            $line =~ s/\[\[.*?\]\]/<ulink url='$link'><citetitle>$linkname<\/citetitle><\/ulink>/;
         } elsif ($link =~ /^mailto:/) {
             $linkname =~ s/^mailto://;
             $line =~ s/\[\[.*?\]\]/<ulink url='$link'><citetitle>$linkname<\/citetitle><\/ulink>/;
         } elsif ($link =~ /^wiki:/) {
             $linkname =~ s/^wiki://;
             $link =~ s/^wiki:/http:\/\/www\.wikipedia\.com\/wiki\.phtml\?title=/;
-            $link =~ s/\ /+/;
+            $link =~ s/\ /_/g;
             $line =~ s/\[\[.*?\]\]/<ulink url='$link'><citetitle>$linkname<\/citetitle><\/ulink>/;
         } elsif ($link =~ /^ldp:/) {
             $linkname =~ s/^ldp://;
@@ -235,28 +252,29 @@ sub ProcessLine {
             if ($nonet) {
                 $line =~ s/\[\[.*?\]\]/<citetitle>$link<\/citetitle>/;
             } else {
-                $tempfile = "/tmp/wt2db-" . $rand;
-                $cmd = "wget -q http://db.linuxdoc.org/cgi-pub/ldp-xml.pl?name=$link -O $tempfile";
-                print "$cmd\n" if ($verbose > 1);
-                $return = system("$cmd");
-                unless ($return) {
-                    open(URL, "$tempfile") || die "wt2db: cannot open temporary file ($!)\n\n";
-                    $link = '';
-                    while ($url_line = <URL>) {
-                        $url_line =~ s/\n//;
-                        if ($url_line =~ /identifier/) {
-                            $link .= $url_line;
-                        }
-                    }
-                    close(URL);
-                    unlink $tempfile;
-                }
-                $link =~ s/^.*?<identifier>//;
-                $link =~ s/<\/identifier>.*?$//;
-                if ($link eq '') {
-                    $linkname = "ERROR: LDP namespace resolution failure on $linkname";
-                }
-                $line =~ s/\[\[.*?\]\]/<ulink url='$link'><citetitle>$linkname<\/citetitle><\/ulink>/;
+                #$tempfile = "/tmp/wt2db-" . $rand;
+                #$cmd = "wget -q http://db.linuxdoc.org/cgi-pub/ldp-xml.pl?name=$link -O $tempfile";
+                #print "Command $cmd\n" if ($verbose > 1);
+                #$return = system("$cmd");
+                #unless ($return) {
+                #    open(URL, "$tempfile") || die "wt2db: cannot open temporary file ($!)\n\n";
+                #    $link = '';
+                #    while ($url_line = <URL>) {
+                #        $url_line =~ s/\n//;
+                #        if ($url_line =~ /identifier/) {
+                #            $link .= $url_line;
+                #        }
+                #    }
+                #    close(URL);
+                #    unlink $tempfile;
+                #}
+                #$link =~ s/^.*?<identifier>//;
+                #$link =~ s/<\/identifier>.*?$//;
+                #if ($link eq '') {
+                #    $linkname = "ERROR: LDP namespace resolution failure on $linkname";
+                #}
+                #$line =~ s/\[\[.*?\]\]/<ulink url='$link'><citetitle>$linkname<\/citetitle><\/ulink>/;
+                $line =~ s/\[\[.*?\]\]/<citetitle>$linkname<\/citetitle>/;
             }
         } elsif ($link =~ /^file:/) {
             $linkname =~ s/^file://;
@@ -300,7 +318,6 @@ sub ProcessLine {
     if ((($line =~ /^<para>/) or
          ($line =~ /^<sect/) or
          ($line =~ /^<screen>/) or
-         ($line =~ /^<screen>/) or
          ($line =~ /^<blockquote>/) or
          ($line =~ /^<literallayout>/) or
          ($line =~ /^<articleinfo>/) or
@@ -309,8 +326,8 @@ sub ProcessLine {
             &closepara;
         $noparatag = $line;
         $noparatag =~ s/^.*?<//;
-        $noparatag =~ s/>.*?$//;
-        $noparaline = $linenumber;
+        $noparatag =~ s/>.*$//;
+	$noparaline = $linenumber;
 
         # screen sections don't embed para tags, but are wrapped in them
         #
@@ -327,11 +344,11 @@ sub ProcessLine {
     if ($noparatag ne '') {
         $temp = $line;
         while ($temp =~ /<$noparatag>/) {
-            $temp =~ s/<?$noparatag>//;
+            $temp =~ s/<$noparatag>//;
             $noparadepth ++;
         }
         while ($temp =~ /<\/$noparatag>/) {
-            $temp =~ s/<?\/$noparatag>//;
+            $temp =~ s/<\/$noparatag>//;
             $noparadepth --;
             if ($noparadepth == 0) {
                 $noparaline = 0;
@@ -369,6 +386,7 @@ sub ProcessLine {
             encode_entities($line);
         }
         $line = "$starttag$line$endtag";
+	chomp($line);
 
     # sect3
     #
@@ -406,6 +424,15 @@ sub ProcessLine {
         }
         $level1 = 1;
 
+    # appendix
+    #
+    } elsif (($line =~ /^<appendix/) or
+    	     ($line =~ /<\/appendix>/)) {
+	&close1;
+
+#    	$appendix = $line;
+#	$appendix =~ s/^\s*?(<appendix[^>]*?>)/$1/;
+    
     # orderedlist
     #
     } elsif ($line =~ /^\s*#/) {
@@ -475,14 +502,17 @@ sub ProcessLine {
         if ($id eq '') {
             $line = "<question><para>$title</para></question>";
         } else {
-            $line = "<question id='$id'><para>$title</para></question>";
+            $line = "<question id='$id'><para id='$id-title'>$title</para></question>";
         }
         unless ($qandaentry) {
             $line = "<qandaentry>\n" . $line;
             $qandaentry = 1;
         }
         if ($qandaset == 0) {
-            $line = "<qandaset defaultlabel='qanda'>\n". $line;
+            $line = "<qandaset defaultlabel='qanda'>\n" .
+	            "<?dbhtml toc='1' ?>\n" .
+#		    "<?dbhtml cell-spacing='1em' cell-padding='1em' ?>" .
+		    $line;
             $qandaset = 1;
         }
 
