@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright (c) 2001, 2002 David Merrill <david@lupercalia.net>.
+# Copyright (c) 2001, 2002, 2003 David Merrill <david@lupercalia.net>.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 use Wt2Db;
 $WT = new Wt2Db;
 
-$buffer = "foo bar
+$buffer = 'foo bar
 
 =Section|section=
 
@@ -57,7 +57,7 @@ paragraph
 
 Foo
 
-";
+';
 
 $outbuf = '';
 
